/*
 *  Contient toutes les classes du jeu PacMan : le joueur, les fantmes et le plateau.
 */

#include <stdio.h>	// printf()
#include "c_pacman.h"	// Classses du jeu et constantes

extern plateau niveau;
extern personnage pacman;

/*****
 *  PERSONNAGE :
 */

personnage::personnage(int x, int y, char car)
{
	this->x = x;
	this->y = y;
	this->car = car;	// Caractre du personnage qui sera affich  l'cran.
	actif = 1;
}

char personnage::get_car()
{
	return car;
}

int personnage::get_x()
{
	return x;
}

int personnage::get_y()
{
	return y;
}

/*
 *  Dplace le personnage (efface aussi son ancien emplacement)
 *  en vrifiant qu'il a le droit d'aller  sa nouvelle position.
 */
int personnage::set_position(int x, int y)
{
	if (actif == 0)
		return 0;

	/* Passer  travers l'cran */
	if (x == -1)			x = LONGEUR_PLATEAU - 1;
	if (x == LONGEUR_PLATEAU)	x = 0;
	if (y == -1)			y = HAUTEUR_PLATEAU - 1;
	if (y == HAUTEUR_PLATEAU)	y = 0;

	/* S'il y a eu dplacement et qu'il est lgitime */
	if ((niveau.emplacement_libre(x, y, car)) &&
	    ((x != this->x) || (y != this->y))) {
		niveau.effacer_caractere(this->x, this->y, car);
		this->x = x;
		this->y = y;
		niveau.afficher_caractere(x, y, car);
		return 1;
	} else
		return 0;
}

/*****
 *  FANTOME :
 */

fantome::fantome(int x, int y, char car) : personnage(x, y, car)
{
	niveau.afficher_caractere(x, y, car);
}

void fantome::associer_pacman(personnage *pacman)
{
	pacman_associe = pacman;
}

void fantome::definir_s_echapper(bool echape)
{
	s_echape = echape;
	definir_chemin();	// Recalculer le chemin en tenant compte de ce paramtre
}

void fantome::definir_actif(bool activer)
{
	if (actif == 0 && activer == 1) {
		actif = activer;
		niveau.afficher_caractere(x, y, car);
	}
	if (actif == 1 && activer == 0) {
		actif = activer;
		niveau.effacer_caractere(x, y, car);
	}
}

void fantome::definir_chemin()
{
	;	// Tenir compte de s_echape.
}

/*
 *  Dplacer de un caractre le fantome selon le chemin prdfini (si inchang).
 */
void fantome::deplacer()
{
	int deplacement_effectue = 0;
	int pmx = pacman_associe->get_x();
	int pmy = pacman_associe->get_y();
	
	int correct = 1;
	if (s_echape)
		correct = -1;

	if (pmx < x)		deplacement_effectue = set_position(x - correct, y);
	else if (pmx > x)	deplacement_effectue = set_position(x + correct, y);
	else if (pmy < y)	deplacement_effectue = set_position(x, y - correct);
	else if (pmy > y)	deplacement_effectue = set_position(x, y + correct);

	if (deplacement_effectue == 0) { // On recommence avec un dplacement prioritaire en y
		if (pmy < y)		deplacement_effectue = set_position(x, y - correct);
		else if (pmy > y)	deplacement_effectue = set_position(x, y + correct);
		else if (pmx < x)	deplacement_effectue = set_position(x - correct, y);
		else if (pmx > x)	deplacement_effectue = set_position(x + correct, y);
	}
}

/*****
 *  PLATEAU :
 */

bool plateau::emplacement_libre(int x, int y, char car)
{
	if (car == '')
		return (niveau[x][y] != MUR) && ( ! get_fantome(x, y));
	else
		return (niveau[x][y] != MUR);	// Un pacman ne doit pas s'arrter sur un fantme
}

void plateau::afficher_caractere(int x, int y, char car)
{
	printf("\033[%d;%dH%c",
		y + DECALLAGE_PLATEAU_Y,
		x + DECALLAGE_PLATEAU_X,
		car);

	if (car == '')
		++fantomes[x][y];
}

void plateau::effacer_caractere(int x, int y, char car)
{
	char caractere = ' ';

	if (niveau[x][y] == POMME)		caractere = '.';
	if (niveau[x][y] == POMME_BONUS)	caractere = ';';

	if (car == '')
		--fantomes[x][y];

	/* Affiche l'ancien caractre s'il y en avait un */
	afficher_caractere(x, y, caractere);
	if (fantomes[x][y])
		printf("\033[%d;%dH%c",
			y + DECALLAGE_PLATEAU_Y,
			x + DECALLAGE_PLATEAU_X,
			'');
}

int plateau::get_at(int x, int y)
{
	return niveau[x][y];
}

void plateau::set_at(int x, int y, char car)
{
	niveau[x][y] = car;
}

int plateau::get_fantome(int x, int y)
{
	return fantomes[x][y];
}
